using System;
using System.Web.UI;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WebEdition;
using System.Collections.ObjectModel;

namespace CSSamples.Samples.MapShapes
{
    public partial class TrackShapeFinishedEvent : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#E5E3DF"));
                Map1.CurrentExtent = new RectangleShape(-97.766, 30.291, -97.755, 30.286);
                Map1.MapUnit = GeographyUnit.DecimalDegree;

                WorldMapKitWmsWebOverlay worldMapKitWmsWebOverlay = new WorldMapKitWmsWebOverlay();
                Map1.CustomOverlays.Add(worldMapKitWmsWebOverlay);

                InMemoryFeatureLayer shapeLayer = new InMemoryFeatureLayer();
                shapeLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimpleCircleStyle(GeoColor.FromArgb(180, 102, 255, 102), 10, GeoColor.StandardColors.DarkGreen, 1);
                shapeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.Green, 4, true);
                shapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(180, 102, 255, 102), GeoColor.StandardColors.DarkGreen, 1);
                shapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                LayerOverlay dynamicOverlay = new LayerOverlay("StaticOverlay");
                dynamicOverlay.IsBaseOverlay = false;
                dynamicOverlay.Layers.Add("SelectedLinksLayer", shapeLayer);
                Map1.CustomOverlays.Add(dynamicOverlay);

                ShapeFileFeatureLayer austinStreetsLabelLayer = new ShapeFileFeatureLayer(Server.MapPath(@"~\SampleData\USA\Austin\austinstreets.shp"));
                austinStreetsLabelLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.LocalRoad1("FENAME");
                austinStreetsLabelLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                austinStreetsLabelLayer.DrawingMarginPercentage = 100;

                LayerOverlay overlay = new LayerOverlay("Roads");
                overlay.Layers.Add("Roads", austinStreetsLabelLayer);
                Map1.CustomOverlays.Add(overlay);
            }
        }

        private void AddFeaturesToSelection(Collection<Feature> features)
        {
            LayerOverlay staticOverlay =
              (LayerOverlay)this.Map1.CustomOverlays["StaticOverlay"];
            InMemoryFeatureLayer selectedLinksLayer =
              (InMemoryFeatureLayer)staticOverlay.Layers["SelectedLinksLayer"];
            selectedLinksLayer.Open();

            foreach (Feature linkFeature in features)
            {
                if (!selectedLinksLayer.InternalFeatures.Contains(linkFeature.Id))
                {
                    selectedLinksLayer.InternalFeatures.Add(linkFeature.Id, linkFeature);
                }
            }

            selectedLinksLayer.Close();
            this.ZoomMapToSelection();
            staticOverlay.Redraw();
        }

        private void ZoomMapToSelection()
        {
            LayerOverlay staticOverlay = (LayerOverlay)this.Map1.CustomOverlays["StaticOverlay"];
            InMemoryFeatureLayer selectedLinksLayer = (InMemoryFeatureLayer)staticOverlay.Layers["SelectedLinksLayer"];

            if (selectedLinksLayer.InternalFeatures.Count > 0)
            {
                this.Map1.CurrentExtent = staticOverlay.GetBoundingBox();
            }
            else
            {
                this.Map1.CurrentExtent = new RectangleShape(-131.22, 55.05, -54.03, 16.91);
            }

            staticOverlay.Redraw();
        }

        protected void buttonDrawPolygon_Click(object sender, ImageClickEventArgs e)
        {
            Map1.EditOverlay.TrackMode = TrackMode.Polygon;
        }

        protected void Map1_TrackShapeFinished(object sender, EventArgs e)
        {
            LayerOverlay roadsOverlay = (LayerOverlay)this.Map1.CustomOverlays["Roads"];
            ShapeFileFeatureLayer roadsLayer = (ShapeFileFeatureLayer)roadsOverlay.Layers["Roads"];
            roadsLayer.Open();

            foreach (Feature feature in this.Map1.EditOverlay.Features)
            {
                Collection<Feature> spatialQueryResults =
                  roadsLayer.QueryTools.GetFeaturesWithin(feature, new string[0]);

                this.AddFeaturesToSelection(spatialQueryResults);
            }

            roadsLayer.Close();

            this.Map1.EditOverlay.TrackMode = TrackMode.None;
            this.Map1.EditOverlay.Features.Clear();
        }

        protected void Button1_Click(object sender, EventArgs e)
        {

            string selectedCounty = "Bunny";

            LayerOverlay roadsOverlay =
              (LayerOverlay)this.Map1.CustomOverlays["Roads"];
            ShapeFileFeatureLayer roadsLayer =
              (ShapeFileFeatureLayer)roadsOverlay.Layers["Roads"];
            roadsLayer.Open();
            Collection<Feature> featuresInCounty =
              roadsLayer.QueryTools.GetFeaturesByColumnValue("FENAME", selectedCounty);
            roadsLayer.Close();

            this.AddFeaturesToSelection(featuresInCounty);
            this.ZoomMapToSelection();
        }
    }
}
